
/*
 * Copyright (C) 2012 - Michael Baudin
 * Copyright (C) 2011 - DIGITEO - Michael Baudin
 * Copyright (C) 2008 - INRIA
 *
 * This file must be used under the terms of the CeCILL.
 * This source file is licensed as described in the file COPYING, which
 * you should have received as part of this distribution.  The terms
 * are also available at
 * http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
 *
 */
 
#ifndef _CDFLIB_H_
#define _CDFLIB_H_

#ifdef _MSC_VER
	#if LIBCDFLIB_EXPORTS 
		#define CDFLIB_IMPORTEXPORT __declspec (dllexport)
	#else
		#define CDFLIB_IMPORTEXPORT __declspec (dllimport)
	#endif
#else
	#define CDFLIB_IMPORTEXPORT
#endif

#undef __BEGIN_DECLS
#undef __END_DECLS
#ifdef __cplusplus
# define __BEGIN_DECLS extern "C" {
# define __END_DECLS }
#else
# define __BEGIN_DECLS /* empty */
# define __END_DECLS /* empty */
#endif

__BEGIN_DECLS

#include "machine.h"

#define min(a,b) ((a) <= (b) ? (a) : (b))
#define max(a,b) ((a) >= (b) ? (a) : (b))
#define TRUE_ (1)
#define FALSE_ (0)

extern double dlamch_(char *, long int);

// If this function is non-NULL, it is used to generate uniform random numbers in [0,1).
// It can be configure with the public function "genrand_randSetFunction".
extern double (* genrand_randfunction)();

// If this function is non-NULL, it is used to generate uniform random integers in [a,b].
// It can be configure with the public function "genrand_randIntegerInRangeSetFunction".
extern double (* genrand_randIntegerInRange)(double *a, double *b);

// If this function is non-NULL, it is used to print messages out.
// It can be configure with the public function "unifrng_msgsetfunction".
// If the function is not configured, the messages are printed in the console.
extern void (* genrand_messagefunction)(char * message);

// Configure the function which generates random numbers uniform in [0,1).
// This function must be called before calling any 
// random number generation function.
void genrand_randSetFunction ( double (* f)(void) );

// Configure the function which generates random integers uniform in [a,b].
// This function must be called before calling any 
// random number generation function.
void genrand_randIntegerInRangeSetFunction ( double (* f)(double *a, double *b) );

// Configure the function which prints messages.
// This function must be called before calling any 
// random number generation function.
void genrand_messageSetFunction ( void (* f)(char * message) );

// Print error messages
void genrand_messageprint(char * msg);

// The function which generates random numbers uniform in [0,1).
double genrand_randgenerate();

// The function which generates random integers uniform in [a,b].
double genrand_generateIntegerInRange(double *a, double *b);

// Standard Exponential Random Variable
double C2F(sexpo) (void);

// Standard Gamma Random Variable
double C2F(sgamma) (double *a);

// Generate Standard Normal Random Variables
double snorm_(void);

// Generate Negative Geometric Random Variables
double igngeom(double p);

// Generate Beta Random Variables
double C2F(genbet) (double *aa, double *bb);

// Generate Chi Random Variables
double C2F(genchi) (double *df);

// Generate Exponential Random Variables
double C2F(genexp) (double *av);

// Generate F Random Variables
double C2F(genf) (double *dfn, double *dfd);

// Generate Gamma Random Variables
double C2F(gengam) (double *a, double *r__);

// Generate Non-Central Chi Random Variables
double C2F(gennch) (double *df, double *xnonc);

// Generate Non-Central F Random Variables
double C2F(gennf) (double *dfn, double *dfd, double *xnonc);

// Generate Normal Random Variables
double C2F(gennor) (double *av, double *sd);

// Generate Multivariate Normal Random Variables
int C2F(genmn) (double *parm, double *x, double *work);

// Generate Multinomial Random Variables
int C2F(genmul) (int *n, double *p, int *ncat, int *ix);

// Generate Random Permutations
int C2F(genprm) (double *array, int *larray);

// Generate Binomial Random Variables
int C2F(ignbin) (int *n, double *pp);

// Generate Negative Binomial Random Variables
int C2F(ignnbn) (int *n, double *p);

// Generate Poisson Random Variables
int C2F(ignpoi) (double *mu);

// Generate Uniform Random Variables
double C2F(genunf)(double *low, double *high);

// SET Generate Multivariate Normal random deviate 
int C2F(setgmn) (double *meanv, double *covm, int *ldcovm, int *p, double *parm, int *ierr);

//
// Elementary Functions
//

// Computes log(1+x) accurately for small x
double C2F(logp1)(double *x);

//  Compute   v = log ( (1 + s)/(1 - s) )  for small s, this is for |s| < SLIM = 0.20
double C2F(lnp1m1)(double *s);

//
// Numerical Linear Algebra
//

// Scalar product of vectors sx and sy
double C2F(sdot) (int *n, double *sx, int *incx, double *sy, int *incy);

// Factors a symmetric positive definite matrix
int C2F(spofa) (double *a, int *lda, int *n, int *info);

__END_DECLS

#endif /** _CDFLIB_H_   **/




